import SwiftUI
import CoreLocation
import MusicKit

/**
 * Weather Vibe iOS App Prototype
 * Location-based music playlist generator
 */

// Main app structure
@main
struct WeatherVibeApp: App {
    var body: some Scene {
        WindowGroup {
            ContentView()
        }
    }
}

// Main content view
struct ContentView: View {
    @StateObject private var locationManager = LocationManager()
    @StateObject private var musicManager = MusicManager()
    @State private var isGeneratingPlaylist = false
    @State private var showSettingsSheet = false
    @State private var showMusicAuthSheet = false
    
    var body: some View {
        NavigationView {
            ZStack {
                // Background gradient based on current weather/time
                weatherGradientView
                
                VStack(spacing: 20) {
                    // Weather information card
                    WeatherCardView(
                        weatherData: locationManager.currentWeather,
                        locationName: locationManager.locationName
                    )
                    .padding(.top, 20)
                    
                    // Visualization of the current vibe
                    VibeVisualizationView(
                        weatherType: locationManager.currentWeather?.weather.first?.main ?? "Clear",
                        recommendationData: musicManager.currentRecommendation
                    )
                    .frame(height: 180)
                    .padding(.horizontal)
                    
                    // Genre tags
                    GenreTagsView(genres: musicManager.currentRecommendation?.genres ?? [])
                    
                    Spacer()
                    
                    // Music service connection status
                    if musicManager.isAuthorized {
                        HStack {
                            Image(systemName: "checkmark.circle.fill")
                                .foregroundColor(.green)
                            Text("Connected to \(musicManager.serviceName)")
                                .font(.subheadline)
                        }
                        .padding(.bottom, 5)
                    }
                    
                    // Generate playlist button
                    Button(action: generatePlaylist) {
                        HStack {
                            Image(systemName: "play.circle.fill")
                                .font(.title2)
                            Text(isGeneratingPlaylist ? "Generating..." : "Create Vibe Playlist")
                                .font(.headline)
                        }
                        .frame(maxWidth: .infinity)
                        .padding()
                        .background(Color.accentColor)
                        .foregroundColor(.white)
                        .cornerRadius(15)
                        .shadow(radius: 5)
                    }
                    .disabled(isGeneratingPlaylist || !locationManager.isLocationAvailable || !musicManager.isAuthorized)
                    .padding(.horizontal)
                    .padding(.bottom, 30)
                }
                .padding()
            }
            .navigationTitle("Weather Vibe")
            .navigationBarTitleDisplayMode(.inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarLeading) {
                    Button(action: { showSettingsSheet = true }) {
                        Image(systemName: "gear")
                    }
                }
                
                ToolbarItem(placement: .navigationBarTrailing) {
                    Button(action: { showMusicAuthSheet = true }) {
                        Label("Music", systemImage: musicManager.isAuthorized ? "music.note" : "music.note.list")
                    }
                }
            }
            .sheet(isPresented: $showSettingsSheet) {
                SettingsView(
                    locationInfluence: $musicManager.locationInfluence,
                    weatherInfluence: $musicManager.weatherInfluence,
                    userPreferencesInfluence: $musicManager.userPreferencesInfluence
                )
            }
            .sheet(isPresented: $showMusicAuthSheet) {
                MusicAuthView(musicManager: musicManager)
            }
            .onAppear {
                locationManager.requestLocationPermission()
                
                // Load music authorization status
                Task {
                    await musicManager.checkAuthorization()
                }
            }
        }
    }
    
    // Gradient background based on weather
    var weatherGradientView: some View {
        let weatherType = locationManager.currentWeather?.weather.first?.main ?? "Clear"
        let isDay = locationManager.isDay
        
        return GeometryReader { geometry in
            ZStack {
                // Base gradient for time of day
                LinearGradient(
                    gradient: Gradient(colors: isDay ? 
                                      [Color(#colorLiteral(red: 0.4745098054, green: 0.8392156959, blue: 0.9764705896, alpha: 1)), Color(#colorLiteral(red: 0.2392156869, green: 0.6745098233, blue: 0.9686274529, alpha: 1))] : 
                                      [Color(#colorLiteral(red: 0.09019608051, green: 0.3019607961, blue: 0.5137255192, alpha: 1)), Color(#colorLiteral(red: 0.1215686277, green: 0.01176470611, blue: 0.4235294163, alpha: 1))]),
                    startPoint: .top,
                    endPoint: .bottom
                )
                .ignoresSafeArea()
                
                // Weather-specific overlay
                weatherGradientOverlay(for: weatherType, isDay: isDay)
                    .ignoresSafeArea()
            }
        }
    }
    
    // Get gradient overlay for specific weather
    func weatherGradientOverlay(for weatherType: String, isDay: Bool) -> some View {
        switch weatherType {
        case "Clear":
            return AnyView(
                isDay ?
                RadialGradient(
                    gradient: Gradient(colors: [Color(#colorLiteral(red: 1, green: 0.9607843137, blue: 0.8117647059, alpha: 0.3)), .clear]),
                    center: .top,
                    startRadius: 0,
                    endRadius: 500
                ).opacity(0.7) :
                StarFieldView()
            )
            
        case "Clouds":
            return AnyView(
                LinearGradient(
                    gradient: Gradient(colors: [Color.white.opacity(0.3), Color.white.opacity(0.1)]),
                    startPoint: .top,
                    endPoint: .bottom
                )
            )
            
        case "Rain", "Drizzle":
            return AnyView(
                ZStack {
                    Color.black.opacity(0.2)
                    RainDropsView()
                }
            )
            
        case "Thunderstorm":
            return AnyView(
                ZStack {
                    Color.black.opacity(0.4)
                    ThunderstormView()
                }
            )
            
        case "Snow":
            return AnyView(
                ZStack {
                    Color.white.opacity(0.3)
                    SnowfallView()
                }
            )
            
        case "Mist", "Fog", "Haze":
            return AnyView(
                ZStack {
                    Color.gray.opacity(0.3)
                    FogView()
                }
            )
            
        default:
            return AnyView(Color.clear)
        }
    }
    
    // Generate a new playlist
    func generatePlaylist() {
        guard locationManager.isLocationAvailable else { return }
        
        isGeneratingPlaylist = true
        
        Task {
            let success = await musicManager.createVibePlaylist(
                latitude: locationManager.currentLocation?.coordinate.latitude ?? 0,
                longitude: locationManager.currentLocation?.coordinate.longitude ?? 0,
                locationName: locationManager.locationName,
                weatherDescription: locationManager.currentWeather?.weather.first?.description ?? "Unknown"
            )
            
            isGeneratingPlaylist = false
            
            // Handle playlist result - could show success/error alert
        }
    }
}

// Weather card view component
struct WeatherCardView: View {
    let weatherData: WeatherData?
    let locationName: String
    
    var body: some View {
        VStack(alignment: .center, spacing: 8) {
            if let weather = weatherData {
                HStack {
                    // Weather icon
                    weatherIcon(for: weather.weather.first?.main ?? "")
                        .font(.system(size: 44))
                    
                    VStack(alignment: .leading) {
                        // Temperature
                        Text("\(Int(weather.main.temp))°C")
                            .font(.system(size: 36, weight: .medium))
                        
                        // Weather description
                        Text(weather.weather.first?.description.capitalized ?? "")
                            .font(.title3)
                            .foregroundColor(.secondary)
                    }
                    
                    Spacer()
                }
                
                Divider()
                
                // Location name
                Text(locationName)
                    .font(.headline)
                    .foregroundColor(.secondary)
            } else {
                // Loading state
                HStack {
                    Image(systemName: "location.circle")
                        .font(.system(size: 44))
                    
                    VStack(alignment: .leading) {
                        Text("Finding location...")
                            .font(.headline)
                        Text("Please wait")
                            .font(.subheadline)
                            .foregroundColor(.secondary)
                    }
                    
                    Spacer()
                }
            }
        }
        .padding()
        .background(Color.white.opacity(0.8))
        .cornerRadius(15)
        .shadow(radius: 5)
        .padding(.horizontal)
    }
    
    // Convert weather type to SF Symbol
    func weatherIcon(for weatherType: String) -> some View {
        let iconName: String
        
        switch weatherType {
        case "Clear": iconName = "sun.max.fill"
        case "Clouds": iconName = "cloud.fill"
        case "Rain": iconName = "cloud.rain.fill"
        case "Drizzle": iconName = "cloud.drizzle.fill"
        case "Thunderstorm": iconName = "cloud.bolt.fill"
        case "Snow": iconName = "cloud.snow.fill"
        case "Mist", "Fog", "Haze": iconName = "cloud.fog.fill"
        default: iconName = "cloud.fill"
        }
        
        return Image(systemName: iconName)
            .foregroundColor(iconColor(for: weatherType))
    }
    
    // Color for each weather type
    func iconColor(for weatherType: String) -> Color {
        switch weatherType {
        case "Clear": return .yellow
        case "Clouds": return .gray
        case "Rain", "Drizzle": return .blue
        case "Thunderstorm": return .purple
        case "Snow": return .white
        case "Mist", "Fog", "Haze": return .gray
        default: return .gray
        }
    }
}

// Vibe visualization - a visual representation of the current recommendation
struct VibeVisualizationView: View {
    let weatherType: String
    let recommendationData: MusicRecommendation?
    
    // Animation properties
    @State private var isAnimating = false
    
    var body: some View {
        ZStack {
            // Background
            RoundedRectangle(cornerRadius: 15)
                .fill(Color.white.opacity(0.8))
                .shadow(radius: 5)
            
            if let recommendation = recommendationData {
                // Energy - particles that move based on tempo and energy
                ForEach(0..<15) { i in
                    Circle()
                        .fill(particleColor(i))
                        .frame(width: particleSize(i, with: recommendation), height: particleSize(i, with: recommendation))
                        .offset(particleOffset(i, with: recommendation))
                        .animation(
                            Animation.spring(response: 0.8, dampingFraction: 0.6)
                                .repeatForever(autoreverses: true)
                                .delay(Double(i) * 0.07),
                            value: isAnimating
                        )
                }
                
                // Texture layer based on acousticness
                textureOverlay(recommendation: recommendation)
                    .mask(
                        RoundedRectangle(cornerRadius: 15)
                            .padding(2)
                    )
                
                // Wave visualization based on valence
                waveVisualization(recommendation: recommendation)
                    .mask(
                        RoundedRectangle(cornerRadius: 15)
                            .padding(2)
                    )
            } else {
                // Placeholder when no data
                Text("Generating vibe...")
                    .font(.headline)
                    .foregroundColor(.secondary)
            }
        }
        .onAppear {
            isAnimating = true
        }
    }
    
    // Colors based on weather and recommendation
    func particleColor(_ index: Int) -> Color {
        guard let recommendation = recommendationData else {
            return .gray
        }
        
        let baseHue = weatherColorHue(weatherType)
        let saturation = 0.7 + (recommendation.energy * 0.3)
        let brightness = 0.5 + (recommendation.valence * 0.5)
        
        // Vary hue slightly for each particle
        let hueVariation = Double(index) * 0.02
        return Color(
            hue: (baseHue + hueVariation).truncatingRemainder(dividingBy: 1.0),
            saturation: saturation,
            brightness: brightness
        )
    }
    
    // Size of particle based on energy
    func particleSize(_ index: Int, with recommendation: MusicRecommendation) -> CGFloat {
        let baseSizes = [12, 8, 15, 10, 7, 9, 14, 11, 8, 13, 10, 9, 12, 8, 11]
        let baseSize = baseSizes[index % baseSizes.count]
        
        // Scale particle size with energy and tempo
        let energyFactor = 0.5 + (recommendation.energy * 1.0)
        let tempoFactor = recommendation.tempo / 120.0 // normalize tempo
        
        return CGFloat(baseSize) * CGFloat(energyFactor * tempoFactor)
    }
    
    // Position of particles based on recommendation
    func particleOffset(_ index: Int, with recommendation: MusicRecommendation) -> CGSize {
        let basePhases = [0.0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6, 1.8, 0.1, 0.3, 0.5, 0.7, 0.9]
        let phase = basePhases[index % basePhases.count]
        
        // The range of the particles' movement is affected by the energy level
        let energyFactor = recommendation.energy
        let spreadX = 60.0 * energyFactor
        let spreadY = 40.0 * energyFactor
        
        // Tempo affects the sin/cos period
        let tempoFactor = recommendation.tempo / 60.0 // normalize to cycles per second
        
        let x = sin(isAnimating ? (2 * .pi * tempoFactor * phase) : 0) * spreadX
        let y = cos(isAnimating ? (2 * .pi * tempoFactor * phase + 0.5) : 0) * spreadY
        
        return CGSize(width: x, height: y)
    }
    
    // Texture overlay based on acousticness
    func textureOverlay(recommendation: MusicRecommendation) -> some View {
        ZStack {
            // More acoustic = more grainy texture
            if recommendation.acousticness > 0.5 {
                Canvas { context, size in
                    // Draw a noise texture
                    let noiseScale = 0.1 + (recommendation.acousticness * 0.4)
                    for x in stride(from: 0, to: size.width, by: 2) {
                        for y in stride(from: 0, to: size.height, by: 2) {
                            let opacity = Double.random(in: 0...0.3) * recommendation.acousticness
                            context.opacity = opacity
                            context.fill(Path(CGRect(x: x, y: y, width: 1, height: 1)), with: .color(.white))
                        }
                    }
                }
            }
            
            // For electronic music, create grid pattern
            if recommendation.acousticness < 0.4 {
                ZStack {
                    // Horizontal lines
                    ForEach(0..<10) { i in
                        Rectangle()
                            .fill(Color.white.opacity(0.1))
                            .frame(height: 1)
                            .offset(y: CGFloat(i * 20) - 100)
                    }
                    
                    // Vertical lines
                    ForEach(0..<12) { i in
                        Rectangle()
                            .fill(Color.white.opacity(0.1))
                            .frame(width: 1)
                            .offset(x: CGFloat(i * 20) - 110)
                    }
                }
            }
        }
        .opacity(0.3)
    }
    
    // Wave visualization based on valence
    func waveVisualization(recommendation: MusicRecommendation) -> some View {
        let lowAmp = 5.0 + (10.0 * recommendation.valence)
        let highAmp = 10.0 + (20.0 * recommendation.valence)
        
        return ZStack {
            // Multiple wave layers with different frequencies
            ForEach(0..<3) { i in
                WaveShape(
                    amplitude: i == 0 ? highAmp : lowAmp,
                    frequency: 0.05 + (Double(i) * 0.02),
                    phase: isAnimating ? Double(i) * 2 : 0
                )
                .fill(
                    Color.white.opacity(0.15 - (Double(i) * 0.05))
                )
                .animation(
                    Animation.linear(duration: 5 / recommendation.tempo * 60)
                        .repeatForever(autoreverses: false),
                    value: isAnimating
                )
                .offset(y: 50 + CGFloat(i * 30))
            }
        }
        .mask(
            // Gradient mask to fade waves
            LinearGradient(
                gradient: Gradient(colors: [.clear, .white, .white, .clear]),
                startPoint: .top,
                endPoint: .bottom
            )
        )
    }
    
    // Map weather type to a hue value
    func weatherColorHue(_ weatherType: String) -> Double {
        switch weatherType {
        case "Clear": return 0.13 // Yellow-orange
        case "Clouds": return 0.62 // Blue-gray
        case "Rain", "Drizzle": return 0.58 // Blue
        case "Thunderstorm": return 0.75 // Purple
        case "Snow": return 0.67 // Light blue
        case "Mist", "Fog", "Haze": return 0.45 // Blue-green
        default: return 0.33 // Green
        }
    }
}

// Wave shape for the visualization
struct WaveShape: Shape {
    var amplitude: Double
    var frequency: Double
    var phase: Double
    
    var animatableData: Double {
        get { phase }
        set { phase = newValue }
    }
    
    func path(in rect: CGRect) -> Path {
        var path = Path()
        let height = rect.height
        let width = rect.width
        
        let midHeight = height / 2
        
        path.move(to: CGPoint(x: 0, y: midHeight))
        
        for x in stride(from: 0, through: width, by: 1) {
            let relativeX = x / width
            let sine = sin(2 * .pi * (Double(relativeX) * frequency + phase))
            let y = midHeight + CGFloat(amplitude * sine)
            path.addLine(to: CGPoint(x: x, y: y))
        }
        
        path.addLine(to: CGPoint(x: width, y: height))
        path.addLine(to: CGPoint(x: 0, y: height))
        path.closeSubpath()
        
        return path
    }
}

// Genre tags component
struct GenreTagsView: View {
    let genres: [String]
    
    var body: some View {
        ScrollView(.horizontal, showsIndicators: false) {
            HStack(spacing: 8) {
                ForEach(genres, id: \.self) { genre in
                    Text(genre.capitalized)
                        .font(.caption)
                        .fontWeight(.medium)
                        .padding(.horizontal, 12)
                        .padding(.vertical, 6)
                        .background(
                            Capsule()
                                .fill(genreColor(genre).opacity(0.2))
                                .overlay(
                                    Capsule()
                                        .strokeBorder(genreColor(genre).opacity(0.5), lineWidth: 1)
                                )
                        )
                        .foregroundColor(genreColor(genre))
                }
            }
            .padding(.horizontal)
        }
    }
    
    // Different colors for different genre categories
    func genreColor(_ genre: String) -> Color {
        let lowerGenre = genre.lowercased()
        
        if lowerGenre.contains("rock") || lowerGenre.contains("punk") || lowerGenre.contains("metal") {
            return .red
        } else if lowerGenre.contains("pop") || lowerGenre.contains("dance") {
            return .pink
        } else if lowerGenre.contains("hip hop") || lowerGenre.contains("rap") || lowerGenre.contains("urban") {
            return .purple
        } else if lowerGenre.contains("electronic") || lowerGenre.contains("techno") || lowerGenre.contains("house") {
            return .blue
        } else if lowerGenre.contains("jazz") || lowerGenre.contains("blues") {
            return .indigo
        } else if lowerGenre.contains("classical") || lowerGenre.contains("piano") {
            return .gray
        } else if lowerGenre.contains("folk") || lowerGenre.contains("acoustic") || lowerGenre.contains("country") {
            return .brown
        } else if lowerGenre.contains("ambient") || lowerGenre.contains("chill") || lowerGenre.contains("lo-fi") {
            return .mint
        } else {
            return .teal
        }
    }
}

// Visual effect views for different weather conditions
struct RainDropsView: View {
    @State private var isAnimating = false
    
    var body: some View {
        Canvas { context, size in
            let dropCount = 100
            
            for i in 0..<dropCount {
                // Randomized drop properties
                let x = Double.random(in: 0...size.width)
                let y = isAnimating ? 
                    ((Double(i) / Double(dropCount) + Double.random(in: 0...1)).truncatingRemainder(dividingBy: 1.0)) * size.height :
                    Double.random(in: 0...size.height)
                
                let length = Double.random(in: 10...25)
                let opacity = Double.random(in: 0.3...0.7)
                
                // Draw the raindrop
                context.opacity = opacity
                context.stroke(
                    Path { path in
                        path.move(to: CGPoint(x: x, y: y))
                        path.addLine(to: CGPoint(x: x, y: y + length))
                    },
                    with: .color(.white),
                    lineWidth: 1
                )
            }
        }
        .onAppear {
            withAnimation(.linear(duration: 1.5).repeatForever(autoreverses: false)) {
                isAnimating = true
            }
        }
    }
}

struct ThunderstormView: View {
    @State private var showLightning = false
    @State private var lightningOpacity = 0.0
    
    var body: some View {
        ZStack {
            // Rain effect
            RainDropsView()
            
            // Lightning flash
            Color.white
                .opacity(lightningOpacity)
                .ignoresSafeArea()
                .onAppear {
                    startLightningEffect()
                }
        }
    }
    
    func startLightningEffect() {
        // Randomize lightning flashes
        DispatchQueue.main.asyncAfter(deadline: .now() + Double.random(in: 3...8)) {
            // Flash sequence
            withAnimation(.easeIn(duration: 0.1)) {
                lightningOpacity = 0.7
            }
            
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                withAnimation(.easeOut(duration: 0.1)) {
                    lightningOpacity = 0.0
                }
                
                // Maybe double flash
                if Bool.random() {
                    DispatchQueue.main.asyncAfter(deadline: .now() + 0.15) {
                        withAnimation(.easeIn(duration: 0.05)) {
                            lightningOpacity = 0.5
                        }
                        
                        DispatchQueue.main.asyncAfter(deadline: .now() + 0.05) {
                            withAnimation(.easeOut(duration: 0.1)) {
                                lightningOpacity = 0.0
                            }
                            
                            // Continue the effect
                            startLightningEffect()
                        }
                    }
                } else {
                    // Continue the effect
                    startLightningEffect()
                }
            }
        }
    }
}

struct SnowfallView: View {
    @State private var isAnimating = false
    
    var body: some View {
        TimelineView(.animation) { timeline in
            Canvas { context, size in
                let snowflakes = 80
                let time = timeline.date.timeIntervalSince1970
                
                for i in 0..<snowflakes {
                    // Snowflake properties
                    let uniqueRandom = ((Double(i) * 0.01) + (time * 0.1)).truncatingRemainder(dividingBy: 1.0)
                    let x = size.width * Double.random(in: 0...1, using: &randomGenerator(seed: i))
                    
                    // Vertical position with slight swaying using sine
                    let baseSpeed = Double.random(in: 10...40, using: &randomGenerator(seed: i * 2))
                    let y = ((time * baseSpeed / 10) + Double(i)).truncatingRemainder(dividingBy: size.height)
                    
                    // Add horizontal swaying
                    let swayAmount = Double.random(in: 2...10, using: &randomGenerator(seed: i * 3))
                    let swayX = sin(time + Double(i)) * swayAmount
                    
                    // Snowflake size
                    let size = Double.random(in: 2...4, using: &randomGenerator(seed: i * 4))
                    
                    // Draw the snowflake
                    context.opacity = Double.random(in: 0.5...0.9, using: &randomGenerator(seed: i * 5))
                    context.fill(
                        Path(ellipseIn: CGRect(x: x + swayX, y: y, width: size, height: size)),
                        with: .color(.white)
                    )
                }
            }
        }
    }
    
    // Deterministic random generator for consistent animation
    func randomGenerator(seed: Int) -> RandomNumberGenerator {
        var randomGenerator = SeededGenerator(seed: seed)
        return randomGenerator
    }
}

struct SeededGenerator: RandomNumberGenerator {
    let seed: Int
    var currentValue: UInt64
    
    init(seed: Int) {
        self.seed = seed
        self.currentValue = UInt64(seed)
    }
    
    mutating func next() -> UInt64 {
        currentValue = currentValue &* 6364136223846793005 &+ 1
        return currentValue
    }
}

struct FogView: View {
    @State private var isAnimating = false
    
    var body: some View {
        TimelineView(.animation) { timeline in
            Canvas { context, size in
                let time = timeline.date.timeIntervalSince1970
                
                // Create fog patches that slowly move
                for i in 0..<5 {
                    let baseX = (time * 2 + Double(i * 100)).truncatingRemainder(dividingBy: size.width + 200) - 100
                    let baseY = Double(i) * size.height / 5
                    
                    // Draw a soft cloud shape
                    let fogPath = Path { path in
                        path.move(to: CGPoint(x: baseX - 50, y: baseY + 20))
                        path.addCurve(
                            to: CGPoint(x: baseX + 250, y: baseY + 20),
                            control1: CGPoint(x: baseX + 50, y: baseY - 30),
                            control2: CGPoint(x: baseX + 150, y: baseY - 30)
                        )
                        path.addCurve(
                            to: CGPoint(x: baseX + 350, y: baseY + 20),
                            control1: CGPoint(x: baseX + 300, y: baseY + 50),
                            control2: CGPoint(x: baseX + 320, y: baseY - 20)
                        )
                        path.addCurve(
                            to: CGPoint(x: baseX - 50, y: baseY + 20),
                            control1: CGPoint(x: baseX + 250, y: baseY + 60),
                            control2: CGPoint(x: baseX + 50, y: baseY + 60)
                        )
                    }
                    
                    context.opacity = 0.3
                    context.fill(fogPath, with: .color(.white))
                }
            }
        }
    }
}

struct StarFieldView: View {
    var body: some View {
        Canvas { context, size in
            // Draw multiple stars
            for _ in 0..<200 {
                let x = Double.random(in: 0...size.width)
                let y = Double.random(in: 0...size.height)
                let starSize = Double.random(in: 0.5...2.0)
                let opacity = Double.random(in: 0.3...1.0)
                
                context.opacity = opacity
                context.fill(
                    Path(ellipseIn: CGRect(x: x, y: y, width: starSize, height: starSize)),
                    with: .color(.white)
                )
            }
        }
    }
}

// Settings view
struct SettingsView: View {
    @Binding var locationInfluence: Double
    @Binding var weatherInfluence: Double
    @Binding var userPreferencesInfluence: Double
    @Environment(\.dismiss) private var dismiss
    
    var body: some View {
        NavigationView {
            Form {
                Section(header: Text("INFLUENCE FACTORS")) {
                    HStack {
                        Text("Location")
                        Spacer()
                        Text("\(Int(locationInfluence * 100))%")
                            .foregroundColor(.secondary)
                    }
                    Slider(value: $locationInfluence, in: 0...1, step: 0.05)
                    
                    HStack {
                        Text("Weather")
                        Spacer()
                        Text("\(Int(weatherInfluence * 100))%")
                            .foregroundColor(.secondary)
                    }
                    Slider(value: $weatherInfluence, in: 0...1, step: 0.05)
                    
                    HStack {
                        Text("Your Music Taste")
                        Spacer()
                        Text("\(Int(userPreferencesInfluence * 100))%")
                            .foregroundColor(.secondary)
                    }
                    Slider(value: $userPreferencesInfluence, in: 0...1, step: 0.05)
                }
                
                Section(header: Text("ABOUT"), footer: Text("Weather Vibe iOS App v1.0")) {
                    HStack {
                        Text("OpenWeatherMap API")
                        Spacer()
                        Text("Connected")
                            .foregroundColor(.green)
                    }
                }
            }
            .navigationTitle("Settings")
            .navigationBarTitleDisplayMode(.inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarTrailing) {
                    Button("Done") {
                        dismiss()
                    }
                }
            }
        }
    }
}

// Music service authorization view
struct MusicAuthView: View {
    @ObservedObject var musicManager: MusicManager
    @Environment(\.dismiss) private var dismiss
    @State private var isRequestingAuth = false
    
    var body: some View {
        NavigationView {
            VStack(spacing: 25) {
                Image(systemName: "music.note.list")
                    .font(.system(size: 70))
                    .foregroundColor(.accentColor)
                    .padding()
                
                Text("Connect Music Service")
                    .font(.title2)
                    .fontWeight(.bold)
                
                Text("Connect to Apple Music or Spotify to create personalized playlists based on your music taste, location, and the current weather.")
                    .multilineTextAlignment(.center)
                    .foregroundColor(.secondary)
                    .padding(.horizontal)
                
                Spacer()
                
                VStack(spacing: 15) {
                    Button(action: connectAppleMusic) {
                        HStack {
                            Image(systemName: "applelogo")
                                .font(.title3)
                            Text("Connect to Apple Music")
                                .fontWeight(.semibold)
                        }
                        .frame(maxWidth: .infinity)
                        .padding()
                        .background(Color.black)
                        .foregroundColor(.white)
                        .cornerRadius(15)
                    }
                    .disabled(isRequestingAuth)
                    
                    Button(action: connectSpotify) {
                        HStack {
                            Image(systemName: "s.circle.fill")
                                .font(.title3)
                            Text("Connect to Spotify")
                                .fontWeight(.semibold)
                        }
                        .frame(maxWidth: .infinity)
                        .padding()
                        .background(Color.green)
                        .foregroundColor(.white)
                        .cornerRadius(15)
                    }
                    .disabled(isRequestingAuth)
                }
                .padding(.horizontal)
                
                if musicManager.isAuthorized {
                    HStack {
                        Image(systemName: "checkmark.circle.fill")
                            .foregroundColor(.green)
                        Text("Connected to \(musicManager.serviceName)")
                            .foregroundColor(.secondary)
                    }
                    .padding()
                    
                    Button("Disconnect") {
                        Task {
                            await musicManager.disconnect()
                        }
                    }
                    .foregroundColor(.red)
                }
                
                Spacer()
            }
            .padding()
            .navigationTitle("Music Service")
            .navigationBarTitleDisplayMode(.inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarTrailing) {
                    Button("Done") {
                        dismiss()
                    }
                }
            }
        }
    }
    
    func connectAppleMusic() {
        isRequestingAuth = true
        
        Task {
            await musicManager.authorizeAppleMusic()
            isRequestingAuth = false
        }
    }
    
    func connectSpotify() {
        isRequestingAuth = true
        
        Task {
            await musicManager.authorizeSpotify()
            isRequestingAuth = false
        }
    }
}

// Location manager
class LocationManager: ObservableObject {
    private let locationManager = CLLocationManager()
    
    @Published var currentLocation: CLLocation?
    @Published var locationName: String = "Finding location..."
    @Published var currentWeather: WeatherData?
    @Published var isLocationAvailable = false
    @Published var isDay = true
    
    init() {
        locationManager.delegate = self
        locationManager.desiredAccuracy = kCLLocationAccuracyHundredMeters
    }
    
    func requestLocationPermission() {
        locationManager.requestWhenInUseAuthorization()
    }
    
    private func fetchWeatherForLocation() {
        guard let location = currentLocation else { return }
        
        // In a real app, this would call your server or directly call the weather API
        // For now, we'll simulate with dummy data
        let mockWeatherTypes = ["Clear", "Clouds", "Rain", "Drizzle", "Thunderstorm", "Snow", "Mist"]
        let randomWeatherIndex = Int.random(in: 0..<mockWeatherTypes.count)
        
        // Create mock weather data
        let weather = WeatherData(
            main: MainWeather(temp: Double.random(in: 5...30)),
            weather: [WeatherCondition(
                main: mockWeatherTypes[randomWeatherIndex],
                description: mockWeatherTypes[randomWeatherIndex].lowercased()
            )],
            sys: WeatherSys(
                sunrise: Date.now.addingTimeInterval(-3600 * 6).timeIntervalSince1970,
                sunset: Date.now.addingTimeInterval(3600 * 6).timeIntervalSince1970
            ),
            dt: Date.now.timeIntervalSince1970
        )
        
        // Update weather and determine if it's day or night
        DispatchQueue.main.async {
            self.currentWeather = weather
            self.isDay = weather.dt > weather.sys.sunrise && weather.dt < weather.sys.sunset
        }
    }
    
    private func reverseGeocode() {
        guard let location = currentLocation else { return }
        
        let geocoder = CLGeocoder()
        geocoder.reverseGeocodeLocation(location) { [weak self] placemarks, error in
            guard let self = self else { return }
            
            if let error = error {
                print("Reverse geocoding error: \(error.localizedDescription)")
                return
            }
            
            if let placemark = placemarks?.first {
                DispatchQueue.main.async {
                    if let city = placemark.locality {
                        self.locationName = city
                    } else if let area = placemark.administrativeArea {
                        self.locationName = area
                    } else {
                        self.locationName = "Unknown location"
                    }
                }
            }
        }
    }
}

// Extend LocationManager to conform to CLLocationManagerDelegate
extension LocationManager: CLLocationManagerDelegate {
    func locationManagerDidChangeAuthorization(_ manager: CLLocationManager) {
        switch manager.authorizationStatus {
        case .authorizedWhenInUse, .authorizedAlways:
            manager.startUpdatingLocation()
            isLocationAvailable = true
        default:
            isLocationAvailable = false
        }
    }
    
    func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        guard let location = locations.last else { return }
        
        // Update location only if it's significantly different
        if currentLocation == nil || currentLocation!.distance(from: location) > 500 {
            currentLocation = location
            reverseGeocode()
            fetchWeatherForLocation()
        }
        
        // Don't need continuous updates for this app
        manager.stopUpdatingLocation()
    }
    
    func locationManager(_ manager: CLLocationManager, didFailWithError error: Error) {
        print("Location error: \(error.localizedDescription)")
        isLocationAvailable = false
    }
}

// Music manager
class MusicManager: ObservableObject {
    @Published var isAuthorized = false
    @Published var serviceName = "Music Service"
    @Published var currentRecommendation: MusicRecommendation?
    
    // Influence factors - how much each aspect affects the final recommendation
    @Published var locationInfluence: Double = 0.4
    @Published var weatherInfluence: Double = 0.3
    @Published var userPreferencesInfluence: Double = 0.3
    
    func checkAuthorization() async {
        // In a real app, check if user is already authorized with Apple Music or Spotify
        let mockAuth = Bool.random()
        if mockAuth {
            await MainActor.run {
                isAuthorized = true
                serviceName = Bool.random() ? "Apple Music" : "Spotify"
            }
        }
    }
    
    func authorizeAppleMusic() async {
        // In a real app, implement MusicKit authorization
        await MainActor.run {
            isAuthorized = true
            serviceName = "Apple Music"
        }
    }
    
    func authorizeSpotify() async {
        // In a real app, implement Spotify OAuth 2.0 flow
        await MainActor.run {
            isAuthorized = true
            serviceName = "Spotify"
        }
    }
    
    func disconnect() async {
        // In a real app, revoke authorization tokens
        await MainActor.run {
            isAuthorized = false
            serviceName = "Music Service"
        }
    }
    
    func createVibePlaylist(latitude: Double, longitude: Double, locationName: String, weatherDescription: String) async -> Bool {
        // This would call your location_music_recommender.js functionality
        // For prototype, we'll generate a mock recommendation
        let mockRecommendation = MusicRecommendation(
            tempo: Double.random(in: 70...140),
            energy: Double.random(in: 0.3...0.9),
            acousticness: Double.random(in: 0.2...0.8),
            valence: Double.random(in: 0.3...0.8),
            genres: mockGenres()
        )
        
        await MainActor.run {
            self.currentRecommendation = mockRecommendation
        }
        
        // Simulate API call delay
        try? await Task.sleep(nanoseconds: 2_000_000_000)
        
        // Success
        return true
    }
    
    private func mockGenres() -> [String] {
        let allGenres = [
            "indie pop", "chill electronic", "acoustic", "pop", "indie folk",
            "jazz", "ambient", "lo-fi", "dance", "hip hop", "r&b", 
            "rock", "alternative", "classical", "electronic", "tropical house"
        ]
        
        // Pick 3-5 random genres
        let count = Int.random(in: 3...5)
        var genres: [String] = []
        
        while genres.count < count {
            if let genre = allGenres.randomElement(), !genres.contains(genre) {
                genres.append(genre)
            }
        }
        
        return genres
    }
}

// Data models

struct WeatherData {
    let main: MainWeather
    let weather: [WeatherCondition]
    let sys: WeatherSys
    let dt: TimeInterval // Current time
}

struct MainWeather {
    let temp: Double
}

struct WeatherCondition {
    let main: String
    let description: String
}

struct WeatherSys {
    let sunrise: TimeInterval
    let sunset: TimeInterval
}

struct MusicRecommendation {
    let tempo: Double // In BPM
    let energy: Double // 0.0 to 1.0
    let acousticness: Double // 0.0 to 1.0
    let valence: Double // 0.0 to 1.0, positiveness
    let genres: [String]
}